@@ -19,7 +19,7 @@ totem_gdk_window_set_invisible_cursor (GdkWindow *window)
 
 	cursor = gdk_cursor_new (GDK_BLANK_CURSOR);
 	gdk_window_set_cursor (window, cursor);
-	gdk_cursor_unref (cursor);
+	g_object_unref (cursor);
 }
 
 void
@@ -29,7 +29,7 @@ totem_gdk_window_set_waiting_cursor (GdkWindow *window)
 
 	cursor = gdk_cursor_new (GDK_WATCH);
 	gdk_window_set_cursor (window, cursor);
-	gdk_cursor_unref (cursor);
+	g_object_unref (cursor);
 
 	gdk_flush ();
 }
@@ -165,50 +165,6 @@ totem_time_to_string_text (gint64 msecs)
 	return string;
 }
 
-typedef struct _TotemPrefSize {
-  gint width, height;
-  gulong sig_id;
-} TotemPrefSize;
-
-static gboolean
-cb_unset_size (gpointer data)
-{
-  GtkWidget *widget = data;
-
-  gtk_widget_queue_resize_no_redraw (widget);
-
-  return FALSE;
-}
-
-static void
-cb_set_preferred_size (GtkWidget *widget, GtkRequisition *req,
-		       gpointer data)
-{
-  TotemPrefSize *size = data;
-
-  req->width = size->width;
-  req->height = size->height;
-
-  g_signal_handler_disconnect (widget, size->sig_id);
-  g_free (size);
-  g_idle_add (cb_unset_size, widget);
-}
-
-void
-totem_widget_set_preferred_size (GtkWidget *widget, gint width,
-				 gint height)
-{
-  TotemPrefSize *size = g_new (TotemPrefSize, 1);
-
-  size->width = width;
-  size->height = height;
-  size->sig_id = g_signal_connect (widget, "size-request",
-				   G_CALLBACK (cb_set_preferred_size),
-				   size);
-
-  gtk_widget_queue_resize (widget);
-}
-
 gboolean
 totem_ratio_fits_screen (GtkWidget *video_widget, int video_width,
 			 int video_height, gfloat ratio)
