@@ -47,28 +47,12 @@
 #define TOTEM_IS_DISC_RECORDER_PLUGIN_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), TOTEM_TYPE_DISC_RECORDER_PLUGIN))
 #define TOTEM_DISC_RECORDER_PLUGIN_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), TOTEM_TYPE_DISC_RECORDER_PLUGIN, TotemDiscRecorderPluginClass))
 
-typedef struct TotemDiscRecorderPluginPrivate         TotemDiscRecorderPluginPrivate;
-
-typedef struct
-{
-	PeasExtensionBase parent;
-
+typedef struct {
 	TotemObject *totem;
 
 	GtkActionGroup *action_group;
 	guint ui_merge_id;
-
-	gboolean enabled;
-} TotemDiscRecorderPlugin;
-
-typedef struct
-{
-	PeasExtensionBaseClass parent_class;
-} TotemDiscRecorderPluginClass;
-
-GType totem_disc_recorder_plugin_get_type	(void) G_GNUC_CONST;
-
-static void peas_activatable_iface_init		(PeasActivatableInterface *iface);
+} TotemDiscRecorderPluginPrivate;
 
 TOTEM_PLUGIN_REGISTER(TOTEM_TYPE_DISC_RECORDER_PLUGIN, TotemDiscRecorderPlugin, totem_disc_recorder_plugin)
 
@@ -96,51 +80,60 @@ totem_disc_recorder_plugin_start_burning (TotemDiscRecorderPlugin *pi,
 {
 	GtkWindow *main_window;
 	GdkScreen *screen;
-	GPtrArray *array;
-	char **args, *xid_str;
+	gchar *command_line;
+	GList *uris;
+	GAppInfo *info;
+	GdkAppLaunchContext *context;
 	GError *error = NULL;
-	gboolean ret;
 	int xid;
 
-	array = g_ptr_array_new ();
-	g_ptr_array_add (array, (gpointer) "brasero");
-	if (copy != FALSE)
-		g_ptr_array_add (array, (gpointer) "-c");
-	else
-		g_ptr_array_add (array, (gpointer) "-r");
-	g_ptr_array_add (array, (gpointer) path);
-
-	main_window = totem_get_main_window (pi->totem);
+	/* Build a command line to use */
+	main_window = totem_get_main_window (pi->priv->totem);
 	screen = gtk_widget_get_screen (GTK_WIDGET (main_window));
-	xid = gdk_x11_drawable_get_xid (GDK_DRAWABLE (gtk_widget_get_window (GTK_WIDGET (main_window))));
-	xid_str = g_strdup_printf ("%d", xid);
-	g_ptr_array_add (array, (gpointer) "-x");
-	g_ptr_array_add (array, xid_str);
+	xid = gdk_x11_window_get_xid (gtk_widget_get_window (GTK_WIDGET (main_window)));
+	g_object_unref (main_window);
 
-	g_ptr_array_add (array, NULL);
-	args = (char **) g_ptr_array_free (array, FALSE);
+	if (copy != FALSE)
+		command_line = g_strdup_printf ("brasero -x %d -c", xid);
+	else
+		command_line = g_strdup_printf ("brasero -x %d -r", xid);
 
-	ret = TRUE;
-	if (gdk_spawn_on_screen (screen, NULL, args, NULL,
-				 G_SPAWN_SEARCH_PATH | G_SPAWN_FILE_AND_ARGV_ZERO,
-				 NULL, NULL, NULL, &error) == FALSE) {
-		if (copy != FALSE) {
-			totem_interface_error (_("The video disc could not be duplicated."),
-					       error->message,
-					       totem_get_main_window (pi->totem));
-		} else {
-			totem_interface_error (_("The movie could not be recorded."),
-					       error->message,
-					       totem_get_main_window (pi->totem));
-		}
-		ret = FALSE;
-		g_error_free (error);
-	}
+	/* Build the app info */
+	info = g_app_info_create_from_commandline (command_line, NULL,
+	                                           G_APP_INFO_CREATE_SUPPORTS_URIS | G_APP_INFO_CREATE_SUPPORTS_STARTUP_NOTIFICATION, &error);
+	g_free (command_line);
 
-	g_free (xid_str);
-	g_free (args);
+	if (error != NULL)
+		goto error;
 
-	return ret;
+	/* Create a launch context and launch it */
+	context = gdk_app_launch_context_new ();
+	gdk_app_launch_context_set_screen (context, screen);
+
+	uris = g_list_prepend (NULL, (gpointer) path);
+	g_app_info_launch_uris (info, uris, G_APP_LAUNCH_CONTEXT (context), &error);
+	g_list_free (uris);
+
+	g_object_unref (info);
+	g_object_unref (context);
+
+	if (error != NULL)
+		goto error;
+
+	return TRUE;
+
+error:
+	main_window = totem_get_main_window (pi->priv->totem);
+
+	if (copy != FALSE)
+		totem_interface_error (_("The video disc could not be duplicated."), error->message, main_window);
+	else
+		totem_interface_error (_("The movie could not be recorded."), error->message, main_window);
+
+	g_error_free (error);
+	g_object_unref (main_window);
+
+	return FALSE;
 }
 
 static char*
@@ -196,7 +189,7 @@ totem_disc_recorder_plugin_write_video_project (TotemDiscRecorderPlugin *pi,
 	if (success < 0)
 		goto error;
 
-	title = totem_get_short_title (pi->totem);
+	title = totem_get_short_title (pi->priv->totem);
 	if (title) {
 		success = xmlTextWriterWriteElement (project,
 						     (xmlChar *) "label",
@@ -215,7 +208,7 @@ totem_disc_recorder_plugin_write_video_project (TotemDiscRecorderPlugin *pi,
 	if (success < 0)
 		goto error;
 
-	uri = totem_get_current_mrl (pi->totem);
+	uri = totem_get_current_mrl (pi->priv->totem);
 	escaped = (unsigned char *) g_uri_escape_string (uri, NULL, FALSE);
 	g_free (uri);
 
@@ -282,7 +275,7 @@ totem_disc_recorder_plugin_burn (GtkAction *action,
 	if (!path) {
 		totem_interface_error (_("The movie could not be recorded."),
 				       error,
-				       totem_get_main_window (pi->totem));
+				       totem_get_main_window (pi->priv->totem));
 		g_free (error);
 		return;
 	}
@@ -299,7 +292,7 @@ totem_disc_recorder_plugin_copy (GtkAction *action,
 {
 	char *mrl;
 
-	mrl = totem_get_current_mrl (pi->totem);
+	mrl = totem_get_current_mrl (pi->priv->totem);
 	if (!g_str_has_prefix (mrl, "dvd:") && !g_str_has_prefix (mrl, "vcd:")) {
 		g_free (mrl);
 		g_assert_not_reached ();
@@ -315,11 +308,11 @@ totem_disc_recorder_file_closed (TotemObject *totem,
 {
 	GtkAction *action;
 
-	action = gtk_action_group_get_action (pi->action_group, "VideoBurnToDisc");
+	action = gtk_action_group_get_action (pi->priv->action_group, "VideoBurnToDisc");
 	gtk_action_set_visible (action, FALSE);
-	action = gtk_action_group_get_action (pi->action_group, "VideoDVDCopy");
+	action = gtk_action_group_get_action (pi->priv->action_group, "VideoDVDCopy");
 	gtk_action_set_visible (action, FALSE);
-	action = gtk_action_group_get_action (pi->action_group, "VideoVCDCopy");
+	action = gtk_action_group_get_action (pi->priv->action_group, "VideoVCDCopy");
 	gtk_action_set_visible (action, FALSE);
 }
 
@@ -328,50 +321,51 @@ totem_disc_recorder_file_opened (TotemObject *totem,
 				 const char *mrl,
 				 TotemDiscRecorderPlugin *pi)
 {
+	TotemDiscRecorderPluginPrivate *priv = pi->priv;
 	GtkAction *action;
 
 	/* Check if that stream is supported by brasero */
 	if (g_str_has_prefix (mrl, "file:")) {
 		/* If the file is supported we can always burn, even if there
 		 * aren't any burner since we can still create an image. */
-		action = gtk_action_group_get_action (pi->action_group, "VideoBurnToDisc");
+		action = gtk_action_group_get_action (priv->action_group, "VideoBurnToDisc");
 		gtk_action_set_visible (action, TRUE);
-		action = gtk_action_group_get_action (pi->action_group, "VideoDVDCopy");
+		action = gtk_action_group_get_action (priv->action_group, "VideoDVDCopy");
 		gtk_action_set_visible (action, FALSE);
-		action = gtk_action_group_get_action (pi->action_group, "VideoVCDCopy");
+		action = gtk_action_group_get_action (priv->action_group, "VideoVCDCopy");
 		gtk_action_set_visible (action, FALSE);
 	}
 	else if (g_str_has_prefix (mrl, "dvd:")) {
-		action = gtk_action_group_get_action (pi->action_group, "VideoBurnToDisc");
+		action = gtk_action_group_get_action (priv->action_group, "VideoBurnToDisc");
 		gtk_action_set_visible (action, FALSE);
-		action = gtk_action_group_get_action (pi->action_group, "VideoDVDCopy");
+		action = gtk_action_group_get_action (priv->action_group, "VideoDVDCopy");
 		gtk_action_set_visible (action, TRUE);
-		action = gtk_action_group_get_action (pi->action_group, "VideoVCDCopy");
+		action = gtk_action_group_get_action (priv->action_group, "VideoVCDCopy");
 		gtk_action_set_visible (action, FALSE);
 	}
 	else if (g_str_has_prefix (mrl, "vcd:")) {
-		action = gtk_action_group_get_action (pi->action_group, "VideoBurnToDisc");
+		action = gtk_action_group_get_action (priv->action_group, "VideoBurnToDisc");
 		gtk_action_set_visible (action, FALSE);
-		action = gtk_action_group_get_action (pi->action_group, "VideoDVDCopy");
+		action = gtk_action_group_get_action (priv->action_group, "VideoDVDCopy");
 		gtk_action_set_visible (action, FALSE);
-		action = gtk_action_group_get_action (pi->action_group, "VideoVCDCopy");
+		action = gtk_action_group_get_action (priv->action_group, "VideoVCDCopy");
 		gtk_action_set_visible (action, TRUE);
 	}
 	else {
-		action = gtk_action_group_get_action (pi->action_group, "VideoBurnToDisc");
+		action = gtk_action_group_get_action (priv->action_group, "VideoBurnToDisc");
 		gtk_action_set_visible (action, FALSE);
-		action = gtk_action_group_get_action (pi->action_group, "VideoDVDCopy");
+		action = gtk_action_group_get_action (priv->action_group, "VideoDVDCopy");
 		gtk_action_set_visible (action, FALSE);
-		action = gtk_action_group_get_action (pi->action_group, "VideoVCDCopy");
+		action = gtk_action_group_get_action (priv->action_group, "VideoVCDCopy");
 		gtk_action_set_visible (action, FALSE);
 	}
 }
 
 static void
-impl_activate (PeasActivatable *plugin, GObject *object)
+impl_activate (PeasActivatable *plugin)
 {
 	TotemDiscRecorderPlugin *pi = TOTEM_DISC_RECORDER_PLUGIN (plugin);
-	TotemObject *totem = TOTEM_OBJECT (object);
+	TotemDiscRecorderPluginPrivate *priv = pi->priv;
 	GtkUIManager *uimanager = NULL;
 	GtkAction *action;
 	char *path;
@@ -390,118 +384,84 @@ impl_activate (PeasActivatable *plugin, GObject *object)
 	g_free (path);
 #endif
 
-	pi->totem = totem;
+	priv->totem = g_object_get_data (G_OBJECT (plugin), "object");
 
-	g_signal_connect (totem,
+	g_signal_connect (priv->totem,
 			  "file-opened",
 			  G_CALLBACK (totem_disc_recorder_file_opened),
 			  plugin);
-	g_signal_connect (totem,
+	g_signal_connect (priv->totem,
 			  "file-closed",
 			  G_CALLBACK (totem_disc_recorder_file_closed),
 			  plugin);
 
 	/* add UI */
-	pi->action_group = gtk_action_group_new ("DiscRecorderActions");
-	gtk_action_group_set_translation_domain (pi->action_group, GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (pi->action_group,
+	priv->action_group = gtk_action_group_new ("DiscRecorderActions");
+	gtk_action_group_set_translation_domain (priv->action_group, GETTEXT_PACKAGE);
+	gtk_action_group_add_actions (priv->action_group,
 				      totem_disc_recorder_plugin_actions,
 				      G_N_ELEMENTS (totem_disc_recorder_plugin_actions),
 				      pi);
 
-	uimanager = totem_get_ui_manager (totem);
-	gtk_ui_manager_insert_action_group (uimanager, pi->action_group, -1);
-	g_object_unref (pi->action_group);
+	uimanager = totem_get_ui_manager (priv->totem);
+	gtk_ui_manager_insert_action_group (uimanager, priv->action_group, -1);
+	g_object_unref (priv->action_group);
 
-	pi->ui_merge_id = gtk_ui_manager_new_merge_id (uimanager);
+	priv->ui_merge_id = gtk_ui_manager_new_merge_id (uimanager);
 
 	gtk_ui_manager_add_ui (uimanager,
-			       pi->ui_merge_id,
+			       priv->ui_merge_id,
 			       "/ui/tmw-menubar/movie/burn-placeholder",
 			       "VideoBurnToDisc",
 			       "VideoBurnToDisc",
 			       GTK_UI_MANAGER_MENUITEM,
 			       TRUE);
-	gtk_ui_manager_add_ui (uimanager,
-			       pi->ui_merge_id,
-			       "/ui/totem-main-popup/burn-placeholder",
-			       "VideoBurnToDisc",
-			       "VideoBurnToDisc",
-			       GTK_UI_MANAGER_MENUITEM,
-			       TRUE);
 
 	gtk_ui_manager_add_ui (uimanager,
-			       pi->ui_merge_id,
+			       priv->ui_merge_id,
 			       "/ui/tmw-menubar/movie/burn-placeholder",
 			       "VideoDVDCopy",
 			       "VideoDVDCopy",
 			       GTK_UI_MANAGER_MENUITEM,
 			       TRUE);
-	gtk_ui_manager_add_ui (uimanager,
-			       pi->ui_merge_id,
-			       "/ui/totem-main-popup/burn-placeholder",
-			       "VideoDVDCopy",
-			       "VideoDVDCopy",
-			       GTK_UI_MANAGER_MENUITEM,
-			       TRUE);
 
 	gtk_ui_manager_add_ui (uimanager,
-			       pi->ui_merge_id,
+			       priv->ui_merge_id,
 			       "/ui/tmw-menubar/movie/burn-placeholder",
 			       "VideoVCDCopy",
 			       "VideoVCDCopy",
 			       GTK_UI_MANAGER_MENUITEM,
 			       TRUE);
-	gtk_ui_manager_add_ui (uimanager,
-			       pi->ui_merge_id,
-			       "/ui/totem-main-popup/burn-placeholder",
-			       "VideoVCDCopy",
-			       "VideoVCDCopy",
-			       GTK_UI_MANAGER_MENUITEM,
-			       TRUE);
 
-	if (!totem_is_paused (totem) && !totem_is_playing (totem)) {
-		action = gtk_action_group_get_action (pi->action_group, "VideoBurnToDisc");
+	if (!totem_is_paused (priv->totem) && !totem_is_playing (priv->totem)) {
+		action = gtk_action_group_get_action (priv->action_group, "VideoBurnToDisc");
 		gtk_action_set_visible (action, FALSE);
-		action = gtk_action_group_get_action (pi->action_group, "VideoDVDCopy");
+		action = gtk_action_group_get_action (priv->action_group, "VideoDVDCopy");
 		gtk_action_set_visible (action, FALSE);
-		action = gtk_action_group_get_action (pi->action_group, "VideoVCDCopy");
+		action = gtk_action_group_get_action (priv->action_group, "VideoVCDCopy");
 		gtk_action_set_visible (action, FALSE);	}
 	else {
 		char *mrl;
 
-		mrl = totem_get_current_mrl (totem);
-		totem_disc_recorder_file_opened (totem, mrl, pi);
+		mrl = totem_get_current_mrl (priv->totem);
+		totem_disc_recorder_file_opened (priv->totem, mrl, pi);
 		g_free (mrl);
 	}
 }
 
 static void
-impl_deactivate (PeasActivatable *plugin, GObject *object)
+impl_deactivate (PeasActivatable *plugin)
 {
 	TotemDiscRecorderPlugin *pi = TOTEM_DISC_RECORDER_PLUGIN (plugin);
-	TotemObject *totem = TOTEM_OBJECT (object);
+	TotemDiscRecorderPluginPrivate *priv = pi->priv;
 	GtkUIManager *uimanager = NULL;
 
-	pi->enabled = FALSE;
+	g_signal_handlers_disconnect_by_func (priv->totem, totem_disc_recorder_file_opened, plugin);
+	g_signal_handlers_disconnect_by_func (priv->totem, totem_disc_recorder_file_closed, plugin);
 
-	g_signal_handlers_disconnect_by_func (totem, totem_disc_recorder_file_opened, plugin);
-	g_signal_handlers_disconnect_by_func (totem, totem_disc_recorder_file_closed, plugin);
+	uimanager = totem_get_ui_manager (priv->totem);
+	gtk_ui_manager_remove_ui (uimanager, priv->ui_merge_id);
+	gtk_ui_manager_remove_action_group (uimanager, priv->action_group);
 
-	uimanager = totem_get_ui_manager (totem);
-	gtk_ui_manager_remove_ui (uimanager, pi->ui_merge_id);
-	gtk_ui_manager_remove_action_group (uimanager, pi->action_group);
-
-	pi->totem = NULL;
+	priv->totem = NULL;
 }
-
-static void
-totem_disc_recorder_plugin_class_init (TotemDiscRecorderPluginClass *klass)
-{
-}
-
-static void
-totem_disc_recorder_plugin_init (TotemDiscRecorderPlugin *pi)
-{
-}
-
