@@ -48,69 +48,14 @@
 #define TOTEM_GALAGO_PLUGIN_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), TOTEM_TYPE_GALAGO_PLUGIN, TotemGalagoPluginClass))
 
 typedef struct {
-	PeasExtensionBase parent;
-
 	guint		handler_id_fullscreen;
 	guint		handler_id_playing;
 	gboolean	idle; /* Whether we're idle */
 	GalagoPerson	*me; /* Me! */
-} TotemGalagoPlugin;
-
-typedef struct {
-	PeasExtensionBaseClass parent_class;
-} TotemGalagoPluginClass;
-
-GType totem_galago_plugin_get_type		(void) G_GNUC_CONST;
-
-static void totem_galago_plugin_dispose		(GObject *object);
-static void totem_galago_plugin_finalize	(GObject *object);
+} TotemGalagoPluginPrivate;
 
 TOTEM_PLUGIN_REGISTER (TOTEM_TYPE_GALAGO_PLUGIN, TotemGalagoPlugin, totem_galago_plugin);
 
-static void
-totem_galago_plugin_class_init (TotemGalagoPluginClass *klass)
-{
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
-	object_class->dispose = totem_galago_plugin_dispose;
-	object_class->finalize = totem_galago_plugin_finalize;
-}
-
-static void
-totem_galago_plugin_init (TotemGalagoPlugin *plugin)
-{
-	if (galago_init (PACKAGE_NAME, GALAGO_INIT_FEED) == FALSE
-	    || galago_is_connected () == FALSE) {
-		g_warning ("Failed to initialise libgalago.");
-		return;
-	}
-
-	/* Get "me" and list accounts */
-	plugin->me = galago_get_me (GALAGO_REMOTE, TRUE);
-}
-
-static void
-totem_galago_plugin_dispose (GObject *object)
-{
-	TotemGalagoPlugin *plugin = TOTEM_GALAGO_PLUGIN (object);
-
-	if (plugin->me != NULL) {
-		g_object_unref (plugin->me);
-		plugin->me = NULL;
-	}
-
-	G_OBJECT_CLASS (totem_galago_plugin_parent_class)->dispose (object);
-}
-
-static void
-totem_galago_plugin_finalize (GObject *object)
-{
-	if (galago_is_connected ())
-		galago_uninit ();
-
-	G_OBJECT_CLASS (totem_galago_plugin_parent_class)->finalize (object);
-}
-
 static void
 totem_galago_set_idleness (TotemGalagoPlugin *plugin, gboolean idle)
 {
@@ -120,11 +65,11 @@ totem_galago_set_idleness (TotemGalagoPlugin *plugin, gboolean idle)
 	if (galago_is_connected () == FALSE)
 		return;
 
-	if (plugin->idle == idle)
+	if (plugin->priv->idle == idle)
 		return;
 
-	plugin->idle = idle;
-	for (account = galago_person_get_accounts (plugin->me, TRUE); account != NULL; account = g_list_next (account)) {
+	plugin->priv->idle = idle;
+	for (account = galago_person_get_accounts (plugin->priv->me, TRUE); account != NULL; account = g_list_next (account)) {
 		presence = galago_account_get_presence ((GalagoAccount *)account->data, TRUE);
 		if (presence != NULL)
 			galago_presence_set_idle (presence, idle, time (NULL));
@@ -152,44 +97,66 @@ property_notify_cb (TotemObject *totem,
 }
 
 static void
-impl_activate (PeasActivatable *plugin, GObject *object)
+impl_activate (PeasActivatable *plugin)
 {
 	TotemGalagoPlugin *pi = TOTEM_GALAGO_PLUGIN (plugin);
-	TotemObject *totem = TOTEM_OBJECT (object);
+	TotemObject *totem;
+
+	if (galago_init (PACKAGE_NAME, GALAGO_INIT_FEED) == FALSE || galago_is_connected () == FALSE) {
+		g_warning ("Failed to initialise libgalago.");
+		return;
+	}
+
+	/* Get "me" and list accounts */
+	pi->priv->me = galago_get_me (GALAGO_REMOTE, TRUE);
+
+	g_object_get (plugin, "object", &totem, NULL);
 
 	if (!galago_is_connected ()) {
 		GtkWindow *window = totem_get_main_window (totem);
 		totem_interface_error (_("Error loading Galago plugin"), _("Could not connect to the Galago daemon."), window);
 		g_object_unref (window);
+		g_object_unref (totem);
 
 		return;
 	}
 
-	pi->handler_id_fullscreen = g_signal_connect (G_OBJECT (totem),
+	pi->priv->handler_id_fullscreen = g_signal_connect (G_OBJECT (totem),
 				"notify::fullscreen",
 				G_CALLBACK (property_notify_cb),
 				pi);
-	pi->handler_id_playing = g_signal_connect (G_OBJECT (totem),
+	pi->priv->handler_id_playing = g_signal_connect (G_OBJECT (totem),
 				"notify::playing",
 				G_CALLBACK (property_notify_cb),
 				pi);
 
 	/* Force setting the current status */
 	totem_galago_update_from_state (totem, pi);
+
+	g_object_unref (totem);
 }
 
 static void
-impl_deactivate (PeasActivatable *plugin, GObject *object)
+impl_deactivate (PeasActivatable *plugin)
 {
 	TotemGalagoPlugin *pi = TOTEM_GALAGO_PLUGIN (plugin);
-	TotemObject *totem = TOTEM_OBJECT (object);
+	TotemObject *totem;
 
 	/* Failed to initialise */
 	if (!galago_is_connected ())
 		return;
 
-	g_signal_handler_disconnect (G_OBJECT (totem), pi->handler_id_fullscreen);
-	g_signal_handler_disconnect (G_OBJECT (totem), pi->handler_id_playing);
+	g_object_get (plugin, "object", &totem, NULL);
+
+	g_signal_handler_disconnect (G_OBJECT (totem), pi->priv->handler_id_fullscreen);
+	g_signal_handler_disconnect (G_OBJECT (totem), pi->priv->handler_id_playing);
+
+	g_object_unref (totem);
 
 	totem_galago_set_idleness (pi, FALSE);
+
+	if (pi->priv->me != NULL)
+		g_object_unref (pi->priv->me);
+
+	galago_uninit ();
 }
