@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 from gi.repository import Peas
 from gi.repository import Gtk
 from gi.repository import Gdk
@@ -242,7 +244,7 @@ class OpenSubtitlesModel(object):
             if self.token:
                 return True
 
-        self.message = _('Could not contact the OpenSubtitles website')
+        self.message = _(u'Could not contact the OpenSubtitles website')
 
         return False
 
@@ -258,12 +260,12 @@ class OpenSubtitlesModel(object):
             try:
                 result = self.server.SearchSubtitles(self.token, [searchdata])
             except xmlrpclib.ProtocolError:
-                self.message = _('Could not contact the OpenSubtitles website')
+                self.message = _(u'Could not contact the OpenSubtitles website')
 
             if result.get('data'):
                 return result['data']
             else:
-                self.message = _('No results found')
+                self.message = _(u'No results found')
 
         return None
 
@@ -275,13 +277,13 @@ class OpenSubtitlesModel(object):
             try:
                 result = self.server.DownloadSubtitles(self.token, [subtitleId])
             except xmlrpclib.ProtocolError:
-                self.message = _('Could not contact the OpenSubtitles website')
+                self.message = _(u'Could not contact the OpenSubtitles website')
 
             if result and result.get('status') == OK200:
                 try:
                     subtitle64 = result['data'][0]['data']
                 except:
-                    self.message = _('Could not contact the OpenSubtitles website')
+                    self.message = _(u'Could not contact the OpenSubtitles website')
                     return None
 
                 import StringIO, gzip, base64
@@ -348,7 +350,7 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
 
         # Set up and populate the languages combobox
         renderer = Gtk.CellRendererText()
-        sorted_languages = Gtk.TreeModelSort.new_with_model(languages)
+        sorted_languages = Gtk.TreeModelSort (model = languages)
         sorted_languages.set_sort_column_id(0, Gtk.SortType.ASCENDING)
         combobox.set_model(sorted_languages)
         combobox.pack_start(renderer, True)
@@ -369,18 +371,18 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
         self.treeview.set_model(self.liststore)
         self.treeview.set_headers_visible(False)
         renderer.set_property('ellipsize', Pango.EllipsizeMode.END)
-        column = Gtk.TreeViewColumn(_("Subtitles"), renderer, text=0)
+        column = Gtk.TreeViewColumn(_(u"Subtitles"), renderer, text=0)
         column.set_resizable(True)
         column.set_expand(True)
         self.treeview.append_column(column)
 	# translators comment:
 	# This is the file-type of the subtitle file detected
-        column = Gtk.TreeViewColumn(_("Format"), renderer, text=1)
+        column = Gtk.TreeViewColumn(_(u"Format"), renderer, text=1)
         column.set_resizable(False)
         self.treeview.append_column(column)
 	# translators comment:
 	# This is a rating of the quality of the subtitle
-        column = Gtk.TreeViewColumn(_("Rating"), renderer, text=2)
+        column = Gtk.TreeViewColumn(_(u"Rating"), renderer, text=2)
         column.set_resizable(False)
         self.treeview.append_column(column)
 
@@ -421,8 +423,9 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
         self.os_action_group = Gtk.ActionGroup(name='OpenSubtitles')
 
         self.action = Gtk.Action(name='opensubtitles',
-                                 label=_('_Download Movie Subtitles...'),
-                                 tooltip=_("Download movie subtitles from OpenSubtitles"))
+                                 label=_(u'_Download Movie Subtitles…'),
+                                 tooltip=_(u"Download movie subtitles from OpenSubtitles"),
+                                 stock_id=None)
 
         self.os_action_group.add_action(self.action)
 
@@ -440,7 +443,7 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
 
         self.manager.ensure_update()
 
-        self.action.connect('activate', self.os_show_dialog, self.totem)
+        self.action.connect('activate', self.os_show_dialog)
 
         self.action.set_sensitive(self.totem.is_playing() and
 				  self.os_check_allowed_scheme() and
@@ -457,7 +460,7 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
         # I think we must use video widget metadata but I don't found a way 
 	# to get this info from python
         filename = self.totem.get_current_mrl()
-        if Gio.content_type_guess(filename, '', 0)[0].split('/')[0] == 'audio':
+        if Gio.content_type_guess(filename, '')[0].split('/')[0] == 'audio':
             return True
         return False
 
@@ -474,13 +477,13 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
         self.apply_button.set_sensitive(False)
 	self.find_button.set_sensitive(False)        
 
-        self.dialog.window.set_cursor(Gdk.Cursor.new(Gdk.CursorType.WATCH))
+        self.dialog.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.WATCH))
 
         thread = SearchThread(self.model)
         thread.start()
         gobject.idle_add(self.os_populate_treeview)
 
-        self.progress.set_text(_('Searching subtitles...'))
+        self.progress.set_text(_(u'Searching subtitles…'))
         gobject.timeout_add(350, self.os_progress_bar_increment, thread)
 
     def os_populate_treeview(self):
@@ -501,14 +504,14 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
 
         self.model.lock.release()
 
-        self.dialog.window.set_cursor(None)
+        self.dialog.get_window().set_cursor(None)
 
         return False
 
     def os_save_selected_subtitle(self, filename=None):
         """
         """
-        self.dialog.window.set_cursor(Gdk.Cursor.new(Gdk.CursorType.WATCH))
+        self.dialog.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.WATCH))
 
         model, rows = self.treeview.get_selection().get_selected_rows()
         if rows:
@@ -538,7 +541,7 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
             thread.start()
             gobject.idle_add(self.os_save_subtitles, filename)
 
-            self.progress.set_text(_('Downloading the subtitles...'))
+            self.progress.set_text(_(u'Downloading the subtitles…'))
             gobject.timeout_add(350, self.os_progress_bar_increment, thread)
         else:
             #warn user!
@@ -564,7 +567,7 @@ class OpenSubtitles(gobject.GObject, Peas.Activatable):
 
         self.model.lock.release()
 
-        self.dialog.window.set_cursor(None)
+        self.dialog.get_window().set_cursor(None)
 
         if suburi:
             self.totem.set_current_subtitle(suburi)
