@@ -30,7 +30,6 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
-#include <gconf/gconf-client.h>
 #include <libpeas/peas-extension-base.h>
 #include <libpeas/peas-object-module.h>
 #include <libpeas/peas-activatable.h>
@@ -48,83 +47,38 @@
 #define TOTEM_IS_SCREENSAVER_PLUGIN_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), TOTEM_TYPE_SCREENSAVER_PLUGIN))
 #define TOTEM_SCREENSAVER_PLUGIN_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), TOTEM_TYPE_SCREENSAVER_PLUGIN, TotemScreensaverPluginClass))
 
-typedef struct
-{
-	PeasExtensionBase parent;
+typedef struct {
 	TotemObject *totem;
 	BaconVideoWidget *bvw;
+	GSettings *settings;
 
 	TotemScrsaver *scr;
 	guint          handler_id_playing;
 	guint          handler_id_metadata;
-	guint          handler_id_gconf;
-} TotemScreensaverPlugin;
-
-typedef struct
-{
-	PeasExtensionBaseClass parent_class;
-} TotemScreensaverPluginClass;
-
-
-GType	totem_screensaver_plugin_get_type		(void) G_GNUC_CONST;
-
-static void totem_screensaver_plugin_finalize		(GObject *object);
+} TotemScreensaverPluginPrivate;
 
 TOTEM_PLUGIN_REGISTER(TOTEM_TYPE_SCREENSAVER_PLUGIN,
 		      TotemScreensaverPlugin,
 		      totem_screensaver_plugin)
 
-static void
-totem_screensaver_plugin_class_init (TotemScreensaverPluginClass *klass)
-{
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
-	object_class->finalize = totem_screensaver_plugin_finalize;
-}
-
-static void
-totem_screensaver_plugin_init (TotemScreensaverPlugin *plugin)
-{
-	plugin->scr = totem_scrsaver_new ();
-	g_object_set (plugin->scr,
-		      "reason", _("Playing a movie"),
-		      NULL);
-}
-
-static void
-totem_screensaver_plugin_finalize (GObject *object)
-{
-	TotemScreensaverPlugin *plugin = TOTEM_SCREENSAVER_PLUGIN (object);
-
-	g_object_unref (plugin->scr);
-
-	G_OBJECT_CLASS (totem_screensaver_plugin_parent_class)->finalize (object);
-}
-
 static void
 totem_screensaver_update_from_state (TotemObject *totem,
 				     TotemScreensaverPlugin *pi)
 {
 	gboolean lock_screensaver_on_audio, can_get_frames;
 	BaconVideoWidget *bvw;
-	GConfClient *gc;
 
 	bvw = BACON_VIDEO_WIDGET (totem_get_video_widget ((Totem *)(totem)));
-	gc = gconf_client_get_default ();
 
-	lock_screensaver_on_audio = gconf_client_get_bool (gc, 
-							   GCONF_PREFIX"/lock_screensaver_on_audio",
-							   NULL);
+	lock_screensaver_on_audio = g_settings_get_boolean (pi->priv->settings, "lock-screensaver-on-audio");
 	can_get_frames = bacon_video_widget_can_get_frames (bvw, NULL);
 
 	if (totem_is_playing (totem) != FALSE && can_get_frames)
-		totem_scrsaver_disable (pi->scr);
+		totem_scrsaver_disable (pi->priv->scr);
 	else if (totem_is_playing (totem) != FALSE && !lock_screensaver_on_audio)
-		totem_scrsaver_disable (pi->scr);
+		totem_scrsaver_disable (pi->priv->scr);
 	else
-		totem_scrsaver_enable (pi->scr);
-
-	g_object_unref (gc);
+		totem_scrsaver_enable (pi->priv->scr);
 }
 
 static void
@@ -138,73 +92,70 @@ property_notify_cb (TotemObject *totem,
 static void
 got_metadata_cb (BaconVideoWidget *bvw, TotemScreensaverPlugin *pi)
 {
-	totem_screensaver_update_from_state (pi->totem, pi);
+	totem_screensaver_update_from_state (pi->priv->totem, pi);
 }
 
 static void
-lock_screensaver_on_audio_changed_cb (GConfClient *client, guint cnxn_id,
-				      GConfEntry *entry, TotemScreensaverPlugin *pi)
+lock_screensaver_on_audio_changed_cb (GSettings *settings, const gchar *key, TotemScreensaverPlugin *pi)
 {
-	totem_screensaver_update_from_state (pi->totem, pi);
+	totem_screensaver_update_from_state (pi->priv->totem, pi);
 }
 
 static void
-impl_activate (PeasActivatable *plugin,
-	       GObject *object)
+impl_activate (PeasActivatable *plugin)
 {
 	TotemScreensaverPlugin *pi = TOTEM_SCREENSAVER_PLUGIN (plugin);
-	TotemObject *totem = TOTEM_OBJECT (object);
-	GConfClient *gc;
+	TotemObject *totem;
 
-	pi->bvw = BACON_VIDEO_WIDGET (totem_get_video_widget (totem));
+	pi->priv->scr = totem_scrsaver_new ();
+	g_object_set (pi->priv->scr,
+	              "reason", _("Playing a movie"),
+	              NULL);
 
-	gc = gconf_client_get_default ();
-	gconf_client_add_dir (gc, GCONF_PREFIX,
-			      GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-	pi->handler_id_gconf = gconf_client_notify_add (gc, GCONF_PREFIX"/lock_screensaver_on_audio",
-							(GConfClientNotifyFunc) lock_screensaver_on_audio_changed_cb,
-							plugin, NULL, NULL);
-	g_object_unref (gc);
+	totem = g_object_get_data (G_OBJECT (plugin), "object");
+	pi->priv->bvw = BACON_VIDEO_WIDGET (totem_get_video_widget (totem));
 
-	pi->handler_id_playing = g_signal_connect (G_OBJECT (totem),
+	pi->priv->settings = g_settings_new (TOTEM_GSETTINGS_SCHEMA);
+	g_signal_connect (pi->priv->settings, "changed::lock-screensaver-on-audio", (GCallback) lock_screensaver_on_audio_changed_cb, plugin);
+
+	pi->priv->handler_id_playing = g_signal_connect (G_OBJECT (totem),
 						   "notify::playing",
 						   G_CALLBACK (property_notify_cb),
 						   pi);
-	pi->handler_id_metadata = g_signal_connect (G_OBJECT (pi->bvw),
+	pi->priv->handler_id_metadata = g_signal_connect (G_OBJECT (pi->priv->bvw),
 						    "got-metadata",
 						    G_CALLBACK (got_metadata_cb),
 						    pi);
 
-	pi->totem = g_object_ref (totem);
+	pi->priv->totem = g_object_ref (totem);
 
 	/* Force setting the current status */
 	totem_screensaver_update_from_state (totem, pi);
 }
 
 static void
-impl_deactivate	(PeasActivatable *plugin,
-		 GObject *object)
+impl_deactivate	(PeasActivatable *plugin)
 {
 	TotemScreensaverPlugin *pi = TOTEM_SCREENSAVER_PLUGIN (plugin);
-	TotemObject *totem = TOTEM_OBJECT (object);
-	GConfClient *gc;
 
-	gc = gconf_client_get_default ();
-	gconf_client_notify_remove (gc, pi->handler_id_gconf);
-	g_object_unref (gc);
+	g_object_unref (pi->priv->settings);
 
-	if (pi->handler_id_playing != 0) {
-		g_signal_handler_disconnect (G_OBJECT (totem), pi->handler_id_playing);
-		pi->handler_id_playing = 0;
+	if (pi->priv->handler_id_playing != 0) {
+		TotemObject *totem;
+		totem = g_object_get_data (G_OBJECT (plugin), "object");
+		g_signal_handler_disconnect (G_OBJECT (totem), pi->priv->handler_id_playing);
+		pi->priv->handler_id_playing = 0;
 	}
-	if (pi->handler_id_metadata != 0) {
-		g_signal_handler_disconnect (G_OBJECT (pi->bvw), pi->handler_id_metadata);
-		pi->handler_id_metadata = 0;
+	if (pi->priv->handler_id_metadata != 0) {
+		g_signal_handler_disconnect (G_OBJECT (pi->priv->bvw), pi->priv->handler_id_metadata);
+		pi->priv->handler_id_metadata = 0;
 	}
 
-	g_object_unref (pi->totem);
-	g_object_unref (pi->bvw);
+	g_object_unref (pi->priv->totem);
+	g_object_unref (pi->priv->bvw);
 
-	totem_scrsaver_enable (pi->scr);
+	totem_scrsaver_enable (pi->priv->scr);
+
+	g_object_unref (pi->priv->scr);
 }
 
