@@ -34,11 +34,11 @@
 #include "totem-playlist.h"
 
 /**
- * TOTEM_GCONF_PREFIX:
+ * TOTEM_GSETTINGS_SCHEMA:
  *
- * The GConf prefix under which all Totem GConf keys are stored.
+ * The GSettings schema under which all Totem settings are stored.
  **/
-#define TOTEM_GCONF_PREFIX "/apps/totem"
+#define TOTEM_GSETTINGS_SCHEMA "org.gnome.totem"
 
 G_BEGIN_DECLS
 
@@ -130,11 +130,6 @@ GQuark totem_remote_setting_quark	(void);
 #define TOTEM_TYPE_REMOTE_SETTING	(totem_remote_setting_get_type())
 #define TOTEM_REMOTE_SETTING		totem_remote_setting_quark ()
 
-GType totem_disc_media_type_get_type	(void);
-GQuark totem_disc_media_type_quark	(void);
-#define TOTEM_TYPE_DISC_MEDIA_TYPE	(totem_disc_media_type_get_type())
-#define TOTEM_DISC_MEDIA_TYPE		totem_disc_media_type_quark ()
-
 #define TOTEM_TYPE_OBJECT              (totem_object_get_type ())
 #define TOTEM_OBJECT(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), totem_object_get_type (), TotemObject))
 #define TOTEM_OBJECT_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), totem_object_get_type (), TotemObjectClass))
@@ -170,9 +165,9 @@ typedef struct _TotemObject TotemObject;
 typedef struct {
 	GObjectClass parent_class;
 
-	void (*file_opened)			(Totem *totem, const char *mrl);
-	void (*file_closed)			(Totem *totem);
-	void (*metadata_updated)		(Totem *totem,
+	void (*file_opened)			(TotemObject *totem, const char *mrl);
+	void (*file_closed)			(TotemObject *totem);
+	void (*metadata_updated)		(TotemObject *totem,
 						 const char *artist,
 						 const char *title,
 						 const char *album,
@@ -191,94 +186,122 @@ void	totem_metadata_updated			(TotemObject *totem,
 						 const char *album,
 						 guint track_num);
 
-void	totem_action_exit			(Totem *totem) G_GNUC_NORETURN;
-void	totem_action_play			(Totem *totem);
-void	totem_action_stop			(Totem *totem);
-void	totem_action_play_pause			(Totem *totem);
-void	totem_action_pause			(Totem *totem);
-void	totem_action_fullscreen_toggle		(Totem *totem);
-void	totem_action_fullscreen			(Totem *totem, gboolean state);
-void	totem_action_next			(Totem *totem);
-void	totem_action_previous			(Totem *totem);
-void	totem_action_seek_time			(Totem *totem, gint64 msec, gboolean accurate);
-void	totem_action_seek_relative		(Totem *totem, gint64 offset, gboolean accurate);
-double	totem_get_volume			(Totem *totem);
-void	totem_action_volume			(Totem *totem, double volume);
-void	totem_action_volume_relative		(Totem *totem, double off_pct);
-void	totem_action_volume_toggle_mute		(Totem *totem);
-gboolean totem_action_set_mrl			(Totem *totem,
+#define totem_action_exit totem_object_action_exit
+void	totem_object_action_exit		(TotemObject *totem) G_GNUC_NORETURN;
+#define totem_action_play totem_object_action_play
+void	totem_object_action_play		(TotemObject *totem);
+#define totem_action_stop totem_object_action_stop
+void	totem_object_action_stop		(TotemObject *totem);
+#define totem_action_play_pause totem_object_action_play_pause
+void	totem_object_action_play_pause		(TotemObject *totem);
+void	totem_action_pause			(TotemObject *totem);
+#define totem_action_fullscreen_toggle totem_object_action_fullscreen_toggle
+void	totem_object_action_fullscreen_toggle	(TotemObject *totem);
+void	totem_action_fullscreen			(TotemObject *totem, gboolean state);
+#define totem_action_next totem_object_action_next
+void	totem_object_action_next		(TotemObject *totem);
+#define totem_action_previous totem_object_action_previous
+void	totem_object_action_previous		(TotemObject *totem);
+#define totem_action_seek_time totem_object_action_seek_time
+void	totem_object_action_seek_time		(TotemObject *totem, gint64 msec, gboolean accurate);
+void	totem_action_seek_relative		(TotemObject *totem, gint64 offset, gboolean accurate);
+#define totem_get_volume totem_object_get_volume
+double	totem_object_get_volume			(TotemObject *totem);
+#define totem_action_volume totem_object_action_volume
+void	totem_object_action_volume		(TotemObject *totem, double volume);
+void	totem_action_volume_relative		(TotemObject *totem, double off_pct);
+void	totem_action_volume_toggle_mute		(TotemObject *totem);
+gboolean totem_action_set_mrl			(TotemObject *totem,
 						 const char *mrl,
 						 const char *subtitle);
-void	totem_action_set_mrl_and_play		(Totem *totem,
+void	totem_action_set_mrl_and_play		(TotemObject *totem,
 						 const char *mrl, 
 						 const char *subtitle);
 
-gboolean totem_action_set_mrl_with_warning	(Totem *totem,
+gboolean totem_action_set_mrl_with_warning	(TotemObject *totem,
 						 const char *mrl,
 						 const char *subtitle,
 						 gboolean warn);
 
-void	totem_action_play_media			(Totem *totem,
+void	totem_action_play_media			(TotemObject *totem,
 						 TotemDiscMediaType type,
 						 const char *device);
 
-void	totem_action_toggle_aspect_ratio	(Totem *totem);
-void	totem_action_set_aspect_ratio		(Totem *totem, int ratio);
-int	totem_action_get_aspect_ratio		(Totem *totem);
-void	totem_action_toggle_controls		(Totem *totem);
-void	totem_action_next_angle			(Totem *totem);
+void	totem_action_toggle_aspect_ratio	(TotemObject *totem);
+void	totem_action_set_aspect_ratio		(TotemObject *totem, int ratio);
+int	totem_action_get_aspect_ratio		(TotemObject *totem);
+void	totem_action_toggle_controls		(TotemObject *totem);
+void	totem_action_next_angle			(TotemObject *totem);
 
-void	totem_action_set_scale_ratio		(Totem *totem, gfloat ratio);
-void    totem_action_error                      (const char *title,
-						 const char *reason,
-						 Totem *totem);
-void    totem_action_play_media_device		(Totem *totem,
+void	totem_action_set_scale_ratio		(TotemObject *totem, gfloat ratio);
+#define totem_action_error totem_object_action_error
+void    totem_object_action_error               (TotemObject *totem,
+						 const char *title,
+						 const char *reason);
+void    totem_action_play_media_device		(TotemObject *totem,
 						 const char *device);
 
-gboolean totem_is_fullscreen			(Totem *totem);
-gboolean totem_is_playing			(Totem *totem);
-gboolean totem_is_paused			(Totem *totem);
-gboolean totem_is_seekable			(Totem *totem);
-GtkWindow *totem_get_main_window		(Totem *totem);
-GtkUIManager *totem_get_ui_manager		(Totem *totem);
-GtkWidget *totem_get_video_widget		(Totem *totem);
-char *totem_get_video_widget_backend_name	(Totem *totem);
-char *totem_get_version				(void);
+gboolean totem_is_fullscreen			(TotemObject *totem);
+#define totem_is_playing totem_object_is_playing
+gboolean totem_object_is_playing		(TotemObject *totem);
+#define totem_is_paused totem_object_is_paused
+gboolean totem_object_is_paused			(TotemObject *totem);
+#define totem_is_seekable totem_object_is_seekable
+gboolean totem_object_is_seekable		(TotemObject *totem);
+#define totem_get_main_window totem_object_get_main_window
+GtkWindow *totem_object_get_main_window		(TotemObject *totem);
+#define totem_get_ui_manager totem_object_get_ui_manager
+GtkUIManager *totem_object_get_ui_manager	(TotemObject *totem);
+GtkWidget *totem_get_video_widget		(TotemObject *totem);
+char *totem_get_video_widget_backend_name	(TotemObject *totem);
+#define totem_get_version totem_object_get_version
+char *totem_object_get_version			(void);
 
 /* Current media information */
-char *	totem_get_short_title			(Totem *totem);
-gint64	totem_get_current_time			(Totem *totem);
+char *	totem_get_short_title			(TotemObject *totem);
+gint64	totem_get_current_time			(TotemObject *totem);
 
 /* Playlist handling */
-guint	totem_get_playlist_length		(Totem *totem);
-void	totem_action_set_playlist_index		(Totem *totem,
+#define totem_get_playlist_length totem_object_get_playlist_length
+guint	totem_object_get_playlist_length	(TotemObject *totem);
+void	totem_action_set_playlist_index		(TotemObject *totem,
 						 guint index);
-int	totem_get_playlist_pos			(Totem *totem);
-char *	totem_get_title_at_playlist_pos		(Totem *totem,
+#define totem_get_playlist_pos totem_object_get_playlist_pos
+int	totem_object_get_playlist_pos		(TotemObject *totem);
+#define totem_get_title_at_playlist_pos totem_object_get_title_at_playlist_pos
+char *	totem_object_get_title_at_playlist_pos	(TotemObject *totem,
 						 guint playlist_index);
-void totem_add_to_playlist_and_play		(Totem *totem,
+#define totem_add_to_playlist_and_play totem_object_add_to_playlist_and_play
+void totem_object_add_to_playlist_and_play	(TotemObject *totem,
 						 const char *uri,
 						 const char *display_name,
 						 gboolean add_to_recent);
-char *  totem_get_current_mrl			(Totem *totem);
-void	totem_set_current_subtitle		(Totem *totem,
+#define totem_get_current_mrl totem_object_get_current_mrl
+char *  totem_object_get_current_mrl		(TotemObject *totem);
+#define totem_set_current_subtitle totem_object_set_current_subtitle
+void	totem_object_set_current_subtitle	(TotemObject *totem,
 						 const char *subtitle_uri);
 /* Sidebar handling */
-void    totem_add_sidebar_page			(Totem *totem,
+#define totem_add_sidebar_page totem_object_add_sidebar_page
+void    totem_object_add_sidebar_page		(TotemObject *totem,
 						 const char *page_id,
 						 const char *title,
 						 GtkWidget *main_widget);
-void    totem_remove_sidebar_page		(Totem *totem,
+#define totem_remove_sidebar_page totem_object_remove_sidebar_page
+void    totem_object_remove_sidebar_page	(TotemObject *totem,
 						 const char *page_id);
 
 /* Remote actions */
-void    totem_action_remote			(Totem *totem,
+#define totem_action_remote totem_object_action_remote
+void    totem_object_action_remote		(TotemObject *totem,
 						 TotemRemoteCommand cmd,
 						 const char *url);
-void	totem_action_remote_set_setting		(Totem *totem,
+#define totem_action_remote_set_setting totem_object_action_remote_set_setting
+void	totem_object_action_remote_set_setting	(TotemObject *totem,
 						 TotemRemoteSetting setting,
 						 gboolean value);
-gboolean totem_action_remote_get_setting	(Totem *totem,
+#define totem_action_remote_get_setting totem_object_action_remote_get_setting
+gboolean totem_object_action_remote_get_setting	(TotemObject *totem,
 						 TotemRemoteSetting setting);
 
 #endif /* __TOTEM_H__ */
